## 在 Acast 中应用 GatewayWorker

[返回主页](../README.md)

### 概述

Acast 提供了对 Workerman 的 GatewayWorker 组件的封装，用于便捷地实现与服务器建立长连接的客户端与客户端之间的通讯。

该服务位于命名空间 `Acast\Socket\Enhanced`。其使用与 `Acast\Socket` 近似，但有几点仍有必要做出说明。

### 提供服务

GatewayWorker 类库由三类 Worker 构成提供服务：Gateway，BusinessWorker 与 Register。

> static function Server::create(string \$app, ?string \$listen = null, ?array \$ssl = null, bool \$businessWorker = true) Server

1. `$listen` 为监听的地址。若为 null，则不会创建 `Gateway`。可以自定义协议，这一点和 `Acast\Socket` 相同。

2. `$businessWorker` 为是否要创建 `BusinessWorker`。

在分布式部署时，同一个服务中 `Gateway` 和 `BusinessWorker` 不一定需要同时运行，此时需要根据情况选择是否创建相关服务。

> function Server::addBusinessWorker() void

以上方法可以单独创建 BusinessWorker。

#### Register

`GatewayWorker\Register` 用于建立 Gateway 与 BusinessWorker 之间的连接，使用基于 TCP 的 text 协议（换行符分包）。

> static function Server::addRegister(string \$name, string \$listen) void

1. `$name` 为 Worker 的名称。

2. `$listen` 为监听的地址和端口，如 "\[::\]:3000"。协议不可自定义。

#### Gateway

`GatewayWorker\Gateway` 主要用于网络 I/O，收到客户端请求后将数据发送给 BusinessWorker 处理，反之，BusinessWorker 要向用户发送数据，也要交由 Gateway。

`Server::workerConfig()` 方法用于配置 `Gateway` 的相关参数。注意 `registerAddress` 的设置。

`Server::event()` 方法用于向 Gateway 注册事件。由于 Gateway 不用于处理业务逻辑，所以 onMessage 事件不可用。同时，尽量避免在 onConnect 和 onStop 事件中加入过多业务逻辑。

`Server::getWorkerProperty()` 方法用于获取当前 Gateway 实例的属性。

#### BusinessWorker

`GatewayWorker\BusinessWorker` 用于处理业务逻辑。

`Server::businessWorkerConfig()` 方法用于配置 BusinessWorker 的相关参数。注意 `$worker->registerAddress` 的设置。

`Server::businessEvent()` 方法用于向 BusinessWorker 注册事件。其中，onMessage 回调用于格式化用户输入并确定路由等参数，其功能与 `Acast\Socket` 中的相同。

注意，由于 BusinessWorker 与网络 I/O 层是分离的，故 onConnect、onMessage、onClose 回调都不会传递连接实例作为第一个参数，而是当前客户端的 client\_id。

`Server::getBusinessWorkerProperty()` 方法用于获取当前 BusinessWorker 实例的属性。

### 与客户端通信

通过 `GatewayWorker\Lib\Gateway` 类中提供的静态方法可以实现与客户端通信。

Acast 还封装了少量的方法，下面会有说明。此外，开发者可以根据实际需要，自行在控制器中封装相应方法。

> static function Controller::send(\$data, \$client\_id = null) bool

向目标客户端发送消息，如果 `$client_id` 为null，则发送给当前用户。

> static function Controller::close(\$data = null, \$client\_id = null) bool

发送消息并关闭客户端连接。

> static function Controller::lock(?callable \$callback = null) void

锁定客户端，之后用户发送的请求不会通过 onMessage 回调以及路由，而是仅执行提供的回调 `$callback` 。若 `$callback` 为 null，则不会执行任何回调。

由于该方法是通过设置 `$_SESSION` 实现的，所以只在当前 tick 有效（事实上 `Gateway` 中所有不需要提供 client\_id 默认为当前用户的方法都仅在当前 tick 有效），所以如果需要在 Timer 中执行 lock 或者 unlock 操作，需要调用 `Gateway::updateSession` 并传入client\_id。

其次，由于 `$_SESSION` 会在当前 tick 结束时被 `serialize()`，而 `Closure` 不可被 `serialize()`，所以 `$callback` 参数不能为 `Closure`。

> static function Controller::unlock() void

解锁客户端。

### 其他

关于更多有关 GatewayWorker 的使用方法，详见 [Workerman 官方文档](http://www.workerman.net/gatewaydoc/)
